@@ -35,6 +35,7 @@ typedef struct Expand {
 #define	XARG		3	/* expanding $*, $@ */
 #define	XCOM		4	/* expanding $() */
 #define XNULLSUB	5	/* "$@" when $# is 0 (don't generate word) */
+#define XSUBMID		6	/* middle of expanding ${} */
 
 /* States used for field splitting */
 #define IFS_WORD	0	/* word has chars (or quotes) */
@@ -199,7 +200,7 @@ expand(const char *cp,	/* input word */
 	doblank = 0;
 	make_magic = 0;
 	word = (f&DOBLANK) ? IFS_WS : IFS_WORD;
-	st_head.next = (SubType *) 0;
+	st_head.next = NULL;
 	st = &st_head;
 
 	while (1) {
@@ -473,7 +474,8 @@ expand(const char *cp,	/* input word */
 			}
 			continue;
 
-		  case XSUB:
+		case XSUB:
+		case XSUBMID:
 			if ((c = *x.str++) == 0) {
 				type = XBASE;
 				if (f&DOBLANK)
@@ -541,9 +543,8 @@ expand(const char *cp,	/* input word */
 		}
 
 		/* check for end of word or IFS separation */
-		if (c == 0 || (!quote && (f & DOBLANK) && doblank && !make_magic
-			       && ctype(c, C_IFS)))
-		{
+		if (c == 0 || (!quote && (f & DOBLANK) && doblank &&
+		    !make_magic && ctype(c, C_IFS))) {
 			/* How words are broken up:
 			 *		   |       value of c
 			 *	  word	   |	ws	nws	0
@@ -555,9 +556,8 @@ expand(const char *cp,	/* input word */
 			 * Note that IFS_NWS/0 generates a word (at&t ksh
 			 * doesn't do this, but POSIX does).
 			 */
-			if (word == IFS_WORD
-			    || (!ctype(c, C_IFSWS) && (c || word == IFS_NWS)))
-			{
+			if (word == IFS_WORD ||
+			    (!ctype(c, C_IFSWS) && c && word == IFS_NWS)) {
 				char *p;
 
 				*dp++ = '\0';
@@ -587,6 +587,17 @@ expand(const char *cp,	/* input word */
 			if (word != IFS_NWS)
 				word = ctype(c, C_IFSWS) ? IFS_WS : IFS_NWS;
 		} else {
+			if (type == XSUB) {
+				if (word == IFS_NWS &&
+				    Xlength(ds, dp) == 0) {
+					char *p;
+
+					*(p = alloc(1, ATEMP)) = '\0';
+					XPput(*wp, p);
+				}
+				type = XSUBMID;
+			}
+
 			/* age tilde_ok info - ~ code tests second bit */
 			tilde_ok <<= 1;
 			/* mark any special second pass chars */
