@@ -795,7 +795,7 @@ j_resume(const char *cp, int bg)
 		}
 		shprintf("%s%s", p->command, p->next ? "| " : null);
 	}
-	shprintf(newline);
+	shprintf('\n');
 	shf_flush(shl_stdout);
 	if (running)
 		j->state = PRUNNING;
@@ -1478,7 +1478,7 @@ j_print(Job *j, int how, struct shf *shf)
 		{
 			if (how == JP_LONG)
 				shf_fprintf(shf, "%s%5d %-20s %s%s", filler, p->pid,
-					space, p->command, p->next ? "|" : null);
+					' ', p->command, p->next ? "|" : null);
 			else if (how == JP_MEDIUM)
 				shf_fprintf(shf, " %s%s", p->command,
 					p->next ? "|" : null);
@@ -1486,7 +1486,7 @@ j_print(Job *j, int how, struct shf *shf)
 		}
 	}
 	if (output)
-		shf_fprintf(shf, newline);
+		shf_fprintf(shf, "\n");
 }
 
 /* Convert % sequence to job
