@@ -5,61 +5,76 @@
 #include "sh.h"
 #include <ctype.h>
 
-
 /* Structure to keep track of the lexing state and the various pieces of info
- * needed for each particular state.
- */
+ * needed for each particular state. */
 typedef struct lex_state Lex_state;
 struct lex_state {
 	int ls_state;
 	union {
-	    /* $(...) */
-	    struct scsparen_info {
-		    int nparen;		/* count open parenthesis */
-		    int csstate; /* XXX remove */
+		/* $(...) */
+		struct scsparen_info {
+			int nparen;	/* count open parenthesis */
+			int csstate;	/* XXX remove */
 #define ls_scsparen ls_info.u_scsparen
-	    } u_scsparen;
+		} u_scsparen;
 
-	    /* $((...)) */
-	    struct sasparen_info {
-		    int nparen;		/* count open parenthesis */
-		    int start;		/* marks start of $(( in output str */
+		/* $((...)) */
+		struct sasparen_info {
+			int nparen;	/* count open parenthesis */
+			int start;	/* marks start of $(( in output str */
 #define ls_sasparen ls_info.u_sasparen
-	    } u_sasparen;
+		} u_sasparen;
 
-	    /* ((...)) */
-	    struct sletparen_info {
-		    int nparen;		/* count open parenthesis */
+		/* ((...)) */
+		struct sletparen_info {
+			int nparen;	/* count open parenthesis */
 #define ls_sletparen ls_info.u_sletparen
-	    } u_sletparen;
+		} u_sletparen;
 
-	    /* `...` */
-	    struct sbquote_info {
-		    int indquotes;	/* true if in double quotes: "`...`" */
+		/* `...` */
+		struct sbquote_info {
+			int indquotes;	/* true if in double quotes: "`...`" */
 #define ls_sbquote ls_info.u_sbquote
-	    } u_sbquote;
+		} u_sbquote;
 
-	    Lex_state *base;		/* used to point to next state block */
+		/* =(...) */
+		struct sletarray_info {
+			int nparen;	/* count open parentheses */
+#define ls_sletarray ls_info.u_sletarray
+		} u_sletarray;
+
+		/* ADELIM */
+		struct sadelim_info {
+			unsigned char nparen;	/* count open parentheses */
+#define SADELIM_BASH	0
+#define SADELIM_MAKE	1
+			unsigned char style;
+			unsigned char delimiter;
+			unsigned char num;
+			unsigned char flags;	/* ofs. into sadelim_flags[] */
+#define ls_sadelim ls_info.u_sadelim
+		} u_sadelim;
+
+		Lex_state *base;	/* used to point to next state block */
 	} ls_info;
 };
 
 typedef struct State_info State_info;
 struct State_info {
-	Lex_state	*base;
-	Lex_state	*end;
+	Lex_state *base;
+	Lex_state *end;
 };
 
-
-static void	readhere ARGS((struct ioword *iop));
-static int	getsc__ ARGS((void));
-static void	getsc_line ARGS((Source *s));
-static int	getsc_bn ARGS((void));
-static char	*get_brace_var ARGS((XString *wsp, char *wp));
-static int	arraysub ARGS((char **strp));
-static const char *ungetsc ARGS((int c));
-static void	gethere ARGS((void));
-static Lex_state *push_state_ ARGS((State_info *si, Lex_state *old_end));
-static Lex_state *pop_state_ ARGS((State_info *si, Lex_state *old_end));
+static void	readhere(struct ioword *);
+static int	getsc__(void);
+static void	getsc_line(Source *);
+static int	getsc_bn(void);
+static char	*get_brace_var(XString *, char *);
+static int	arraysub(char **);
+static const char *ungetsc(int);
+static void	gethere(void);
+static Lex_state *push_state_(State_info *, Lex_state *);
+static Lex_state *pop_state_(State_info *, Lex_state *);
 
 static int backslash_skip;
 static int ignore_backslash_newline;
@@ -84,8 +99,6 @@ static int ignore_backslash_newline;
 			    state = statep->ls_state; \
 			} while (0)
 
-
-
 /*
  * Lexical analyzer
  *
@@ -101,14 +114,13 @@ yylex(int cf)
 	State_info state_info;
 	int c, state;
 	XString ws;		/* expandable output word */
-	char *wp;	/* output word pointer */
+	char *wp;		/* output word pointer */
 	char *sp, *dp;
 	int c2;
 
-
-  Again:
+ Again:
 	states[0].ls_state = -1;
-	states[0].ls_info.base = (Lex_state *) 0;
+	states[0].ls_info.base = NULL;
 	statep = &states[1];
 	state_info.base = states;
 	state_info.end = &states[STATE_BSIZE];
@@ -120,13 +132,6 @@ yylex(int cf)
 
 	if (cf&ONEWORD)
 		state = SWORD;
-#ifdef KSH
-	else if (cf&LETEXPR) {
-		*wp++ = OQUOTE;	 /* enclose arguments in (double) quotes */
-		state = SLETPAREN;	
-		statep->ls_sletparen.nparen = 0;
-	}
-#endif /* KSH */
 	else {		/* normal lexing */
 		state = (cf & HEREDELIM) ? SHEREDELIM : SBASE;
 		while ((c = getsc()) == ' ' || c == '\t')
@@ -153,11 +158,10 @@ yylex(int cf)
 	{
 		Xcheck(ws, wp);
 		switch (state) {
-		  case SBASE:
+		case SBASE:
 			if (c == '[' && (cf & (VARASN|ARRAYVAR))) {
 				*wp = EOS; /* temporary */
-				if (is_wdvarname(Xstring(ws, wp), FALSE))
-				{
+				if (is_wdvarname(Xstring(ws, wp), FALSE)) {
 					char *p, *tmp;
 
 					if (arraysub(&tmp)) {
@@ -188,47 +192,40 @@ yylex(int cf)
 			/* fall through.. */
 		  Sbase2:	/* doesn't include *(...|...) pattern (*+?@!) */
 			switch (c) {
-			  case '\\':
+			case '\\':
 				c = getsc();
-#ifdef OS2
-				if (isalnum(c) || c=='_') {
-					*wp++ = CHAR, *wp++ = '\\';
-					*wp++ = CHAR, *wp++ = c;
-				} else 
-#endif
 				if (c) /* trailing \ is lost */
 					*wp++ = QCHAR, *wp++ = c;
 				break;
-			  case '\'':
+			case '\'':
 				*wp++ = OQUOTE;
 				ignore_backslash_newline++;
 				PUSH_STATE(SSQUOTE);
 				break;
-			  case '"':
+			case '"':
 				*wp++ = OQUOTE;
 				PUSH_STATE(SDQUOTE);
 				break;
-			  default:
+			default:
 				goto Subst;
 			}
 			break;
 
-		  Subst:
+ Subst:
 			switch (c) {
-			  case '\\':
+			case '\\':
 				c = getsc();
 				switch (c) {
-				  case '\\':
-				  case '$': case '`':
+				case '"':
+					if ((cf & HEREDOC))
+						goto heredocquote;
+					/* FALLTHROUGH */
+				case '\\':
+				case '$': case '`':
 					*wp++ = QCHAR, *wp++ = c;
 					break;
-				  case '"':
-					if ((cf & HEREDOC) == 0) {
-						*wp++ = QCHAR, *wp++ = c;
-						break;
-					}
-					/* FALLTROUGH */
-				  default:
+				default:
+ heredocquote:
 					Xcheck(ws, wp);
 					if (c) { /* trailing \ is lost */
 						*wp++ = CHAR, *wp++ = '\\';
@@ -237,7 +234,7 @@ yylex(int cf)
 					break;
 				}
 				break;
-			  case '$':
+			case '$':
 				c = getsc();
 				if (c == '(') /*)*/ {
 					c = getsc();
@@ -245,7 +242,7 @@ yylex(int cf)
 						PUSH_STATE(SASPAREN);
 						statep->ls_sasparen.nparen = 2;
 						statep->ls_sasparen.start =
-							Xsavepos(ws, wp);
+						    Xsavepos(ws, wp);
 						*wp++ = EXPRSUB;
 					} else {
 						ungetsc(c);
@@ -295,7 +292,7 @@ yylex(int cf)
 					ungetsc(c);
 				}
 				break;
-			  case '`':
+			case '`':
 				PUSH_STATE(SBQUOTE);
 				*wp++ = COMSUB;
 				statep->ls_sbquote.indquotes = 0;
@@ -320,7 +317,7 @@ yylex(int cf)
 			}
 			break;
 
-		  case SSQUOTE:
+		case SSQUOTE:
 			if (c == '\'') {
 				POP_STATE();
 				*wp++ = CQUOTE;
@@ -329,7 +326,7 @@ yylex(int cf)
 				*wp++ = QCHAR, *wp++ = c;
 			break;
 
-		  case SDQUOTE:
+		case SDQUOTE:
 			if (c == '"') {
 				POP_STATE();
 				*wp++ = CQUOTE;
@@ -337,47 +334,47 @@ yylex(int cf)
 				goto Subst;
 			break;
 
-		  case SCSPAREN: /* $( .. ) */
+		case SCSPAREN: /* $( .. ) */
 			/* todo: deal with $(...) quoting properly
 			 * kludge to partly fake quoting inside $(..): doesn't
 			 * really work because nested $(..) or ${..} inside
 			 * double quotes aren't dealt with.
 			 */
 			switch (statep->ls_scsparen.csstate) {
-			  case 0: /* normal */
+			case 0: /* normal */
 				switch (c) {
-				  case '(':
+				case '(':
 					statep->ls_scsparen.nparen++;
 					break;
-				  case ')':
+				case ')':
 					statep->ls_scsparen.nparen--;
 					break;
-				  case '\\':
+				case '\\':
 					statep->ls_scsparen.csstate = 1;
 					break;
-				  case '"':
+				case '"':
 					statep->ls_scsparen.csstate = 2;
 					break;
-				  case '\'':
+				case '\'':
 					statep->ls_scsparen.csstate = 4;
 					ignore_backslash_newline++;
 					break;
 				}
 				break;
 
-			  case 1: /* backslash in normal mode */
-			  case 3: /* backslash in double quotes */
+			case 1: /* backslash in normal mode */
+			case 3: /* backslash in double quotes */
 				--statep->ls_scsparen.csstate;
 				break;
 
-			  case 2: /* double quotes */
+			case 2: /* double quotes */
 				if (c == '"')
 					statep->ls_scsparen.csstate = 0;
 				else if (c == '\\')
 					statep->ls_scsparen.csstate = 3;
 				break;
 
-			  case 4: /* single quotes */
+			case 4: /* single quotes */
 				if (c == '\'') {
 					statep->ls_scsparen.csstate = 0;
 					ignore_backslash_newline--;
@@ -391,10 +388,10 @@ yylex(int cf)
 				*wp++ = c;
 			break;
 
-		  case SASPAREN: /* $(( .. )) */
+		case SASPAREN: /* $(( .. )) */
 			/* todo: deal with $((...); (...)) properly */
 			/* XXX should nest using existing state machine
-			 *     (embed "..", $(...), etc.) */
+			 * (embed "..", $(...), etc.) */
 			if (c == '(')
 				statep->ls_sasparen.nparen++;
 			else if (c == ')') {
@@ -414,23 +411,22 @@ yylex(int cf)
 						 * parsing a $(..) expression
 						 */
 						s = Xrestpos(ws, wp,
-						     statep->ls_sasparen.start);
+						    statep->ls_sasparen.start);
 						memmove(s + 1, s, wp - s);
 						*s++ = COMSUB;
 						*s = '('; /*)*/
 						wp++;
 						statep->ls_scsparen.nparen = 1;
 						statep->ls_scsparen.csstate = 0;
-						state = statep->ls_state
-							= SCSPAREN;
-						
+						state = statep->ls_state =
+						    SCSPAREN;
 					}
 				}
 			}
 			*wp++ = c;
 			break;
 
-		  case SBRACE:
+		case SBRACE:
 			/*{*/
 			if (c == '}') {
 				POP_STATE();
@@ -440,7 +436,7 @@ yylex(int cf)
 				goto Sbase2;
 			break;
 
-		  case STBRACE:
+		case STBRACE:
 			/* Same as SBRACE, except (,|,) treated specially */
 			/*{*/
 			if (c == '}') {
@@ -451,23 +447,23 @@ yylex(int cf)
 				goto Sbase2;
 			break;
 
-		  case SBQUOTE:
+		case SBQUOTE:
 			if (c == '`') {
 				*wp++ = 0;
 				POP_STATE();
 			} else if (c == '\\') {
 				switch (c = getsc()) {
-				  case '\\':
-				  case '$': case '`':
+				case '\\':
+				case '$': case '`':
 					*wp++ = c;
 					break;
-				  case '"':
+				case '"':
 					if (statep->ls_sbquote.indquotes) {
 						*wp++ = c;
 						break;
 					}
 					/* fall through.. */
-				  default:
+				default:
 					if (c) { /* trailing \ is lost */
 						*wp++ = '\\';
 						*wp++ = c;
@@ -478,7 +474,7 @@ yylex(int cf)
 				*wp++ = c;
 			break;
 
-		  case SWORD:	/* ONEWORD */
+		case SWORD:	/* ONEWORD */
 			goto Subst;
 
 #ifdef KSH
@@ -531,17 +527,17 @@ yylex(int cf)
 			}
 			break;
 
-		  case SHEREDQUOTE:	/* " in <<,<<- delimiter */
+		case SHEREDQUOTE:	/* " in <<,<<- delimiter */
 			if (c == '"') {
 				*wp++ = CQUOTE;
 				state = statep->ls_state = SHEREDELIM;
 			} else {
 				if (c == '\\') {
 					switch (c = getsc()) {
-					  case '\\': case '"':
-					  case '$': case '`':
+					case '\\': case '"':
+					case '$': case '`':
 						break;
-					  default:
+					default:
 						if (c) { /* trailing \ lost */
 							*wp++ = CHAR;
 							*wp++ = '\\';
@@ -556,7 +552,7 @@ yylex(int cf)
 
 		}
 	}
-Done:
+ Done:
 	Xcheck(ws, wp);
 	if (statep != &states[1])
 		/* XXX figure out what is missing */
@@ -600,9 +596,9 @@ Done:
 				ungetsc(c2);
 		}
 
-		iop->name = (char *) 0;
-		iop->delim = (char *) 0;
-		iop->heredoc = (char *) 0;
+		iop->name = NULL;
+		iop->delim = NULL;
+		iop->heredoc = NULL;
 		Xfree(ws, wp);	/* free word */
 		yylval.iop = iop;
 		return REDIR;
@@ -654,18 +650,24 @@ Done:
 
 	*wp++ = EOS;		/* terminate word */
 	yylval.cp = Xclose(ws, wp);
-	if (state == SWORD
-#ifdef KSH
-		|| state == SLETPAREN
-#endif /* KSH */
-		)	/* ONEWORD? */
+	if (state == SWORD)	/* ONEWORD? */
 		return LWORD;
 	ungetsc(c);		/* unget terminator */
 
 	/* copy word to unprefixed string ident */
-	for (sp = yylval.cp, dp = ident; dp < ident+IDENT && (c = *sp++) == CHAR; )
-		*dp++ = *sp++;
-	/* Make sure the ident array stays '\0' paded */
+	sp = yylval.cp;
+	dp = ident;
+	if ((cf & HEREDELIM) && (sp[1] == '<'))
+		while (dp < ident+IDENT) {
+			if ((c = *sp++) == CHAR)
+				*dp++ = *sp++;
+			else if ((c != OQUOTE) && (c != CQUOTE))
+				break;
+		}
+	else
+		while (dp < ident+IDENT && (c = *sp++) == CHAR)
+			*dp++ = *sp++;
+	/* Make sure the ident array stays '\0' padded */
 	memset(dp, 0, (ident+IDENT) - dp + 1);
 	if (c != EOS)
 		*ident = '\0';	/* word is not unquoted */
@@ -683,7 +685,7 @@ Done:
 		if ((cf & ALIAS) && (p = transitional_tsearch(&aliases.root, ident))
 		    && (p->flag & ISSET))
 		{
-			register Source *s;
+			Source *s;
 
 			for (s = source; s->type == SALIAS; s = s->next)
 				if (s->u.tblp == p)
@@ -703,9 +705,9 @@ Done:
 }
 
 static void
-gethere()
+gethere(void)
 {
-	register struct ioword **p;
+	struct ioword **p;
 
 	for (p = heres; p < herep; p++)
 		readhere(*p);
@@ -717,10 +719,9 @@ gethere()
  */
 
 static void
-readhere(iop)
-	struct ioword *iop;
+readhere(struct ioword *iop)
 {
-	register int c;
+	int c;
 	char *volatile eof;
 	char *eofp;
 	int skiptabs;
@@ -797,11 +798,9 @@ yyerror(const char *fmt, ...)
  */
 
 Source *
-pushs(type, areap)
-	int type;
-	Area *areap;
+pushs(int type, Area *areap)
 {
-	register Source *s;
+	Source *s;
 
 	s = (Source *) alloc(sizeof(Source), areap);
 	s->type = type;
@@ -824,43 +823,43 @@ pushs(type, areap)
 static int
 getsc__()
 {
-	register Source *s = source;
-	register int c;
+	Source *s = source;
+	int c;
 
 	while ((c = *s->str++) == 0) {
 		s->str = NULL;		/* return 0 for EOF by default */
 		switch (s->type) {
-		  case SEOF:
+		case SEOF:
 			s->str = null;
 			return 0;
 
-		  case SSTDIN:
-		  case SFILE:
+		case SSTDIN:
+		case SFILE:
 			getsc_line(s);
 			break;
 
-		  case SWSTR:
+		case SWSTR:
 			break;
 
-		  case SSTRING:
+		case SSTRING:
 			break;
 
-		  case SWORDS:
+		case SWORDS:
 			s->start = s->str = *s->u.strv++;
 			s->type = SWORDSEP;
 			break;
 
-		  case SWORDSEP:
+		case SWORDSEP:
 			if (*s->u.strv == NULL) {
-				s->start = s->str = newline;
+				s->start = s->str = "\n";
 				s->type = SEOF;
 			} else {
-				s->start = s->str = space;
+				s->start = s->str = " ";
 				s->type = SWORDS;
 			}
 			break;
 
-		  case SALIAS:
+		case SALIAS:
 			if (s->flags & SF_ALIASEND) {
 				/* pass on an unused SF_ALIAS flag */
 				source = s->next;
@@ -903,7 +902,7 @@ getsc__()
 			}
 			continue;
 
-		  case SREREAD:
+		case SREREAD:
 			if (s->start != s->ugbuf) /* yuck */
 				afree(s->u.freeme, ATEMP);
 			source = s = s->next;
@@ -923,8 +922,7 @@ getsc__()
 }
 
 static void
-getsc_line(s)
-	Source *s;
+getsc_line(Source *s)
 {
 	char *xp = Xstring(s->xs, xp);
 	int interactive = Flag(FTALKING) && s->type == SSTDIN;
@@ -958,8 +956,7 @@ getsc_line(s)
 			nread = 0;
 		xp[nread] = '\0';
 		xp += nread;
-	}
-	else
+	} else
 #endif /* EDIT */
 	{
 		if (interactive) {
@@ -999,8 +996,7 @@ getsc_line(s)
 	 */
 	source = s;
 #ifdef KSH
-	if (have_tty && ksh_tmout)
-	{
+	if (have_tty && ksh_tmout) {
 		ksh_tmout_state = TMOUT_EXECUTING;
 		alarm(0);
 	}
@@ -1032,7 +1028,7 @@ getsc_line(s)
 #endif /* HISTORY */
 	}
 	if (interactive)
-		set_prompt(PS2, (Source *) 0);
+		set_prompt(PS2, NULL);
 }
 
 void
@@ -1152,13 +1148,13 @@ get_brace_var(XString *wsp, char *wp)
 		c = getsc();
 		/* State machine to figure out where the variable part ends. */
 		switch (state) {
-		  case PS_INITIAL:
+		case PS_INITIAL:
 			if (c == '#') {
 				state = PS_SAW_HASH;
 				break;
 			}
 			/* fall through.. */
-		  case PS_SAW_HASH:
+		case PS_SAW_HASH:
 			if (isalpha(c) || c=='_')
 				state = PS_IDENT;
 			else if (isdigit(c))
@@ -1168,7 +1164,7 @@ get_brace_var(XString *wsp, char *wp)
 			else
 				state = PS_END;
 			break;
-		  case PS_IDENT:
+		case PS_IDENT:
 			if (!isalnum(c) && c!='_') {
 				state = PS_END;
 				if (c == '[') {
@@ -1186,14 +1182,14 @@ get_brace_var(XString *wsp, char *wp)
 				}
 			}
 			break;
-		  case PS_NUMBER:
+		case PS_NUMBER:
 			if (!isdigit(c))
 				state = PS_END;
 			break;
-		  case PS_VAR1:
+		case PS_VAR1:
 			state = PS_END;
 			break;
-		  case PS_END: /* keep gcc happy */
+		case PS_END: /* keep gcc happy */
 			break;
 		}
 		if (state == PS_END) {
@@ -1213,13 +1209,12 @@ get_brace_var(XString *wsp, char *wp)
  * (Returned string double null terminated)
  */
 static int
-arraysub(strp)
-	char **strp;
+arraysub(char **strp)
 {
 	XString ws;
 	char	*wp;
 	char	c;
-	int 	depth = 1;	/* we are just past the initial [ */
+	int	depth = 1;	/* we are just past the initial [ */
 
 	Xinit(ws, wp, 32, ATEMP);
 
@@ -1241,8 +1236,7 @@ arraysub(strp)
 
 /* Unget a char: handles case when we are already at the start of the buffer */
 static const char *
-ungetsc(c)
-	int c;
+ungetsc(int c)
 {
 	if (backslash_skip)
 		backslash_skip--;
@@ -1266,7 +1260,7 @@ ungetsc(c)
 
 /* Called to get a char that isn't a \newline sequence. */
 static int
-getsc_bn ARGS((void))
+getsc_bn(void)
 {
 	int c, c2;
 
@@ -1294,11 +1288,9 @@ getsc_bn ARGS((void))
 }
 
 static Lex_state *
-push_state_(si, old_end)
-	State_info *si;
-	Lex_state *old_end;
+push_state_(State_info *si, Lex_state *old_end)
 {
-	Lex_state	*new = alloc(sizeof(Lex_state) * STATE_BSIZE, ATEMP);
+	Lex_state *new = alloc(sizeof(Lex_state) * STATE_BSIZE, ATEMP);
 
 	new[0].ls_info.base = old_end;
 	si->base = &new[0];
@@ -1307,9 +1299,7 @@ push_state_(si, old_end)
 }
 
 static Lex_state *
-pop_state_(si, old_end)
-	State_info *si;
-	Lex_state *old_end;
+pop_state_(State_info *si, Lex_state *old_end)
 {
 	Lex_state *old_base = si->base;
 
@@ -1318,5 +1308,5 @@ pop_state_(si, old_end)
 
 	afree(old_base, ATEMP);
 
-	return si->base + STATE_BSIZE - 1;;
+	return si->base + STATE_BSIZE - 1;
 }
