@@ -665,7 +665,7 @@ const	struct tokeninfo {
 #endif /* KSH */
 	/* and some special cases... */
 	{ "newline",	'\n',	FALSE },
-	{ 0 }
+	{ 0, 0, 0 }
 };
 
 void
